'''
Created on 11.6.2009

Copyright 2009, Mikko Talkara

    This file is part of Uppaal Reader.

    Uppaal Reader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Uppaal Reader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Uppaal Reader.  If not, see <http://www.gnu.org/licenses/>.
'''
import getopt
import sys
import os.path
import re

class ParseError(IOError):
    pass

class Parser:
    """Parser class for parsing Uppaal Reader trace files"""
    def __init__(self, input_file, comment_file=''):        
        self.input_f = input_file
        self.comment_f = comment_file
        self.showStateInfo = True
        self.showStateComments = True
        self.showTransComments = True
        if self.comment_f == '':
            self.showTransComments = False
            
        self.output = ""
        self.iterNum = 0
        self.allLines = open(self.input_f, 'r').readlines()
 
    def parse(self):
        state_list = []
        
        def CheckComments(search_str):
            """
            If line containing '->' is found from search_str, search 
            comments matching the state IDs and names from
            .xml file.
                                
             Example: bypass._id47->bypass.Idle
             
             state IDs/names: id47, Idle                        
             """
            comments = ''
            commentList = open(self.comment_f, 'r').readlines() # read file to list of lines
            
            try:
                it = iter(commentList)
                next = it.next()
                
                # check if line contains '->'
                reo = re.compile('->')
                if (reo.search(search_str)) != None:
                    # search for state IDs/names
                    m1 = re.search('(?<=\.)(\w+)(?=->)', search_str)
                    id1 = m1.group(0)
                    m2 = re.search('(?<=\.)(\w+)\Z', search_str)
                    id2 = m2.group(0)
                    while next != None:
                        if id1 in next and id2 in next:
                            statement = next.strip()    # save current statement for further analysis
                            next = it.next()
                            
                            if next.startswith('Transition:') == False \
                            and next.startswith('Location:') == False \
                            and next != None:
                                comments += '\tComments for "' + statement + '":\n'

                            """
                            Iterate until all comments of specific id have been through.
                                                        
                            Example:
                            
                            Location: (Id = id2)
                            State.c: 1006        <-- comment
                            Location: (Id = id3)                    
                            """
                            while next.startswith('Transition:') == False \
                            and next.startswith('Location:') == False \
                            and next != None:
                                comments += '\t\t\t\t' + next
                                next =  it.next()
                        next = it.next()
                else:
                    raise StopIteration

            except StopIteration:                       
                    return comments
            #-----------------------------------------------------------------
        def CheckStateComments(state):
            """
            state - a name string of state for which comments are searched
            
            Method checks whether comments for given state can be found by iterating through
            temporary comment file tmp_file.txt"""
            comments = ''
            # read file to list of lines
            comment_list = open(self.comment_f, 'r').readlines()
            
            try:
                it = iter(comment_list)
                next = it.next()
                while next != None:
                    if next.startswith('Location:'):
                        tmp = next
                        next = it.next()
                        while (next.startswith('Location:') == False and
                               next.startswith('Transition:') == False):
                            if state in tmp:
                                comments += next
                                tmp = next
                            next = it.next()
                    else:
                        next = it.next()
                    
            except StopIteration:
                return comments
            #------------------------------------------------------------------
        # Parse method main body        
        '''Iterate through the trace file parsing state and transition 
        information of interest.
        
        Structure of the trace file is following...
        
        State:                    // (or just 'State', without colon)
        ( something )
        something
        
        Transition:
            data { something }
        
        State:
        '''
        try:
            it = iter(self.allLines)
            next = it.next()
            while next != None:
                if next == 'State:\n' or next == 'State\n' and \
                self.showStateInfo == True:
                    next = it.next()
                    while next.startswith('(') == False:
                        next = it.next() 
                    state_list = next.replace('(','').replace(')','').split()
                    self.iterNum += 1
                    
                elif next == 'Transitions:\n':
                    next = it.next()
                    if self.showStateInfo == True:
                                self.output += 'Transition #' + str(self.iterNum) + '\n'
                                self.output += '\tStates:\n'
                    
                    statement_list = []     # temporary statement list
                    # stop iteration at empty line
                    while next != '\n':
                        # parse transition state                        
                        statement = next.split('{')[0]     # acknowledge only left side of the brackets
                        statement_list.append(statement.strip())
                        next = it.next()
          
                    trans_list = []
                    # add transition states to transition states list
                    for s in statement_list:
                        tmp_states = s.strip().split('->')
                        trans_list.append(tmp_states[0])
                        trans_list.append(tmp_states[1])
                             
                    # show states and comments
                    if self.showStateInfo == True:
                        """
                        Check if transition states are found at 
                        state_list and show found states"""
                        for state in trans_list:
                            if state in state_list:
                                self.output += '\t\t' + state + '\n'
                                if self.showStateComments == True:
                                    state_comments = CheckStateComments(state)
                                    if state_comments != "":
                                       self.output += '\t\t\t' + str(state_comments)
                        self.output += '\n'
                    
                    # show transitions and comments
                    self.output += '\t' + 'Transitions:\n'
                    for statement in statement_list:
                        self.output += '\t\t' + statement + '\n'                         
                        # if true check if location contains comments 
                        if self.showTransComments == True:
                            trans_comments = CheckComments(statement)
                            if trans_comments != "":
                                self.output += '\t' + str(trans_comments)
                    self.output += '\n'

                next = it.next()
                
        except StopIteration:
            """If parser has not found any transitions, file structure is 
            probably incorrect."""
            if self.iterNum == 0:
                raise ParseError, 'Invalid file structure. No transition data found.'
            else:
                return self.output