'''
Created on Jul 16, 2009

Copyright 2009, Mikko Talkara

    This file is part of Uppaal Reader.

    Uppaal Reader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Uppaal Reader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Uppaal Reader.  If not, see <http://www.gnu.org/licenses/>.
'''
import unittest
import os
import TraceParser
import tempfile
import XmlParser
import gui
import wx
from gui import GUI
from xml.sax import make_parser
from xml.sax import ContentHandler


class Test(unittest.TestCase):

    def testParsing(self):
        """Tests that parsing produces correct output at specified input 
        string
        
        Note: Use editor option to display whitespace characters to simplify 
        debugging"""
        fd,filename = tempfile.mkstemp()
        f = open(filename, 'w+')
        
        string = \
'''State:
( something )
something

Transitions:
\tdata { something }

State:
( something )
something

Transitions:
\tdata { something }
\tdata { something }

State:
( something )
something'''
        
        f.write(string)
        f.close()
        
        expected = \
'''#1 Transitions:
\tdata

#2 Transitions:
\tdata
\tdata

'''

        parser = TraceParser.Parser(filename)
        output = parser.parse()
        self.assertEquals(output, expected)
        os.remove(filename)
    
    def testParseError(self):
        '''Tests TraceParser for invalid input file structure.'''
        self.filename = os.path.join(os.getcwd(), 'data/UPS5.xml')
        parser = TraceParser.Parser(self.filename)
        self.assertRaises(TraceParser.ParseError, parser.parse)
        
    def testModelParsing(self):
        """Tests that XmlParser produces expected output"""
        fd, xml_filename = tempfile.mkstemp()
        fd, tmp_filename = tempfile.mkstemp()
        f = open(xml_filename, 'w+')
        xml_string = \
"""
<nta>
    <someElement>
        something
    </someElement>
    <location id="id0" x="0" y="0">
        <committed/>
    </location>
    <location id="id1" x="1000" y="1000">
        <name x="100" y="100">something</name>
    <label kind="comments">State.c: 1006</label>
    </location>
    <transition>
        <source ref="id2"/>
        <target ref="id3"/>
    </transition>
</nta>
"""

        expected = \
"""Location: (Id = id0)
Location: (Id = id1)
State.c: 1006
Transition: (id2 -> id3)
"""
        
        f.write(xml_string)
        f.close()
        parser = make_parser()
        curlHandler = XmlParser.ModelHandler(tmp_filename)
        parser.setContentHandler(curlHandler)
        parser.parse(open(xml_filename))
        parsed_s = open(tmp_filename, 'r').read()
        self.assertEquals(expected, parsed_s)
        os.remove(xml_filename)
        os.remove(tmp_filename)
    
    def testOnSave(self):
        save_file = 'testOnSave.txt'
               
        app = wx.App()
        testGUI = GUI(None, -1, 'Save test')
        GUI.OnSave(testGUI, None, save_file)
        
        #test that file is created
        self.assertTrue(os.path.isfile(save_file))
        os.remove(save_file)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()